use quote::quote;
use syn::Field;

use crate::is_type;

/// Generate struct fields for Partial Model
pub fn partial_code_field(field: &Field) -> proc_macro2::TokenStream {
    let field_name = &field.ident.as_ref().unwrap();
    let field_type = &field.ty;
    let field_name_str = field_name.to_string();

    // Keep _id
    if field_name_str == "_id" {
        return quote! {
            pub _id: String,
        };
    }

    // Leave Option<T> alone
    if is_type(field_type, "Option") {
        return quote! {
            pub #field_name: #field_type,
        };
    }

    // Turn every field into Option<T>
    quote! {
        pub #field_name: Option<#field_type>,
    }
}
