use syn::{Type, TypePath};

/// Get inner type. Example: Returns `T` for `Option<T>`.
pub fn extract_inner_type<'a>(ty: &'a Type, parent: &'a str) -> Option<&'a Type> {
    if let Type::Path(type_path) = ty {
        if type_path.path.segments.len() == 1 {
            let segment = &type_path.path.segments[0];
            if segment.ident == parent {
                if let syn::PathArguments::AngleBracketed(ref args) = segment.arguments {
                    if args.args.len() == 1 {
                        if let syn::GenericArgument::Type(ref inner_type) = args.args[0] {
                            return Some(inner_type);
                        }
                    }
                }
            }
        }
    }
    None
}

pub fn type_path(ty: &syn::Type) -> TypePath {
    if let syn::Type::Path(type_path) = ty {
        return type_path.clone();
    }
    unreachable!();
}

pub fn is_one_of_type(ty: &syn::Type, t: &[&str]) -> bool {
    for typ in t {
        if is_type(ty, typ) {
            return true;
        }
    }
    false
}

pub fn is_type(ty: &syn::Type, t: &str) -> bool {
    let type_path = type_path(ty);
    let id = type_path.path.segments.first().unwrap().ident.to_string();
    id == t
}
