use quote::quote;
use syn::Field;

use crate::{extract_inner_type, is_type};

/// Generate code for the update fn of models
pub fn update_code_field(field: &Field) -> proc_macro2::TokenStream {
    let field_name = &field.ident.as_ref().unwrap();
    let field_type = &field.ty;
    let field_name_str = field_name.to_string();

    // Never update _id
    if field_name_str == "_id" {
        return quote! {};
    }

    if is_type(field_type, "Historic") {
        let inner_field_type = extract_inner_type(field_type, "Historic").unwrap();
        if is_type(inner_field_type, "Vec") {
            // Custom code Historic<Vec<T>>
            return quote! {
                mongod::update_historic_vec!(self, obj, #field_name_str, #field_name, update);
            };
        }

        // Code for Historic<T>
        return quote! {
            mongod::update_historic_str!(self, obj, #field_name_str, #field_name, update);
        };
    }

    if is_type(field_type, "Option") {
        let inner_field_type = extract_inner_type(field_type, "Option").unwrap();

        if is_type(inner_field_type, "Historic") {
            let sub_inner_field_type = extract_inner_type(inner_field_type, "Historic").unwrap();
            if is_type(sub_inner_field_type, "Reference") {
                // Code for Option<Historic<Reference>>
                return quote! {
                    mongod::update_historic_ref_option!(self, obj, #field_name_str, #field_name, update);
                };
            }
        }

        // Code for Option<T>
        return quote! {
            mongod::update_value_option!(self, obj, #field_name_str, #field_name, update, #inner_field_type);
        };
    }

    // Code for T
    quote! {
        mongod::update_value!(self, obj, #field_name_str, #field_name, update, #field_type);
    }
}
